use crate::{
    engine::{render::RenderCtx, BG},
    FG,
};

use super::geometry::{Position, Size};

pub trait Object {
    fn position(&self) -> Position;
    fn size(&self) -> Size;
    fn display(&self, ctx: &mut RenderCtx<'_, '_>) {
        let Position { x, y } = self.position();
        let Size { width, height } = self.size();
        ctx.rect(x, y, width, height, FG);
    }
}

pub trait MovingObject: Object {
    fn previous_pos(&self) -> Position;
    fn update_pos(&mut self, x: u32, y: u32);
    fn move_obj(&mut self, x: i32, y: i32) {
        let Position { x: cur_x, y: cur_y } = self.position();

        self.update_pos(
            cur_x.saturating_add_signed(x),
            cur_y.saturating_add_signed(y),
        )
    }

    // TODO: damage
    fn draw_move(&self, ctx: &mut RenderCtx<'_, '_>) {
        let Position {
            x: prev_x,
            y: prev_y,
        } = self.previous_pos();
        let Size { width, height } = self.size();

        ctx.rect(prev_x, prev_y, width, height, BG);
        self.display(ctx);
    }

    fn move_constrained(
        &mut self,
        x: i32,
        y: i32,
        constraint_start: Position,
        constraint_end: Position,
    ) {
        let Position { x: cur_x, y: cur_y } = self.position();
        let Size { width, height } = self.size();

        let new_x = if cur_x.saturating_add_signed(x) <= constraint_start.x {
            constraint_start.x
        } else if (cur_x + width.saturating_add_signed(y)) > constraint_end.x {
            constraint_end.x - width
        } else {
            cur_x.saturating_add_signed(x)
        };

        let new_y = if cur_y.saturating_add_signed(y) < constraint_start.y {
            constraint_start.y
        } else if (cur_y + height.saturating_add_signed(y)) >= constraint_end.y {
            constraint_end.y - height
        } else {
            cur_y.saturating_add_signed(y)
        };
        self.update_pos(new_x, new_y)
    }
}
