use std::rc::Rc;

use engine::{
    objs::{geometry::Position, primitive_shapes::MovingRect},
    Engine,
};
use winit::{
    event::KeyEvent,
    keyboard::{Key, KeyCode, NamedKey, PhysicalKey, SmolStr},
};

const GAME_SIZE: (u32, u32) = (1200, 800);
const BORDER_WIDTH: u32 = 10;
const PADDLE_SIZE: (u32, u32) = (BORDER_WIDTH, 100);
const PADDLE_OFFSET: u32 = 20;
const PADDLE_SPEED: i32 = 10;
const FG: u32 = 0xebdbb2;
const BG: u32 = 0x282828;

fn main() {
    let mut engine = Engine::new();

    // left paddle
    let _left_paddle_id = engine.insert_into_world(Rc::new(MovingRect::new(
        BORDER_WIDTH + PADDLE_OFFSET,
        GAME_SIZE.1 / 2 - PADDLE_SIZE.1 / 2,
        PADDLE_SIZE.0,
        PADDLE_SIZE.1,
    )));
    let _right_paddle_id = engine.insert_into_world(Rc::new(MovingRect::new(
        GAME_SIZE.0 - (BORDER_WIDTH + PADDLE_OFFSET + PADDLE_SIZE.0),
        GAME_SIZE.1 / 2 - PADDLE_SIZE.1 / 2,
        PADDLE_SIZE.0,
        PADDLE_SIZE.1,
    )));

    const LEFT_PADDLE_CONSTRAINTS: (Position, Position) = (
        Position::new(BORDER_WIDTH + PADDLE_OFFSET, BORDER_WIDTH),
        Position::new(BORDER_WIDTH * 2 + PADDLE_OFFSET, GAME_SIZE.1 - BORDER_WIDTH),
    );
    const RIGHT_PADDLE_CONSTRAINTS: (Position, Position) = (
        Position::new(
            GAME_SIZE.0 - (BORDER_WIDTH + PADDLE_OFFSET + PADDLE_SIZE.0),
            BORDER_WIDTH,
        ),
        Position::new(
            GAME_SIZE.0 - (BORDER_WIDTH + PADDLE_OFFSET),
            GAME_SIZE.1 - BORDER_WIDTH,
        ),
    );

    engine
        .register_handler(
            engine::KeyEventType::KeyDownFrameUpdate(KeyCode::KeyW),
            |world, timer| {
                Rc::get_mut(world.get_mut(0)).unwrap().move_constrained(
                    0,
                    -PADDLE_SPEED,
                    LEFT_PADDLE_CONSTRAINTS.0,
                    LEFT_PADDLE_CONSTRAINTS.1,
                )
            },
        )
        .register_handler(
            engine::KeyEventType::KeyDownFrameUpdate(KeyCode::KeyS),
            |world, timer| {
                Rc::get_mut(world.get_mut(0)).unwrap().move_constrained(
                    0,
                    PADDLE_SPEED,
                    LEFT_PADDLE_CONSTRAINTS.0,
                    LEFT_PADDLE_CONSTRAINTS.1,
                )
            },
        )
        .register_handler(
            engine::KeyEventType::KeyDownFrameUpdate(KeyCode::KeyK),
            |world, timer| {
                Rc::get_mut(world.get_mut(1)).unwrap().move_constrained(
                    0,
                    -PADDLE_SPEED,
                    RIGHT_PADDLE_CONSTRAINTS.0,
                    RIGHT_PADDLE_CONSTRAINTS.1,
                )
            },
        )
        .register_handler(
            engine::KeyEventType::KeyDownFrameUpdate(KeyCode::KeyJ),
            |world, timer| {
                Rc::get_mut(world.get_mut(1)).unwrap().move_constrained(
                    0,
                    PADDLE_SPEED,
                    RIGHT_PADDLE_CONSTRAINTS.0,
                    RIGHT_PADDLE_CONSTRAINTS.1,
                )
            },
        )
        .set_resize_fn(|ctx, world, timer| {
            // top
            ctx.rect(0, 0, GAME_SIZE.0, BORDER_WIDTH, FG);
            // left
            ctx.rect(
                0,
                BORDER_WIDTH,
                BORDER_WIDTH,
                GAME_SIZE.1 - BORDER_WIDTH * 2,
                FG,
            );
            // right
            ctx.rect(
                GAME_SIZE.0 - BORDER_WIDTH,
                BORDER_WIDTH,
                BORDER_WIDTH,
                GAME_SIZE.1 - BORDER_WIDTH * 2,
                FG,
            );
            // bottom
            ctx.rect(0, GAME_SIZE.1 - BORDER_WIDTH, GAME_SIZE.0, 10, FG);
        })
        .set_render_fn(|ctx, world, timer| {
            Rc::get_mut(world.get_mut(0)).unwrap().draw_move(ctx);
            Rc::get_mut(world.get_mut(1)).unwrap().draw_move(ctx);
        })
        .run();
}

mod engine;
